@@ -3,7 +3,7 @@ package Crypt::JWT;
 use strict;
 use warnings;
 
-our $VERSION = '0.026';
+our $VERSION = '0.031';
 
 use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw(decode_jwt encode_jwt)] );
@@ -12,7 +12,7 @@ our @EXPORT = qw();
 
 use Carp;
 use Crypt::Misc qw(decode_b64u encode_b64u);
-use JSON::MaybeXS qw(decode_json encode_json);
+use JSON qw(decode_json encode_json);
 use Crypt::PK::RSA;
 use Crypt::PK::ECC;
 use Crypt::PK::Ed25519;
@@ -133,7 +133,7 @@ sub _kid_lookup {
   return $found if $found->{kty} eq 'oct' && $alg =~ /^(HS|dir|PBES2-HS|A)/;
   return $found if $found->{kty} eq 'OKP' && $alg =~ /^(EdDSA|ECDH-ES)/;
   return $found if $found->{kty} eq 'EC'  && $alg =~ /^(ES|EC)/;
-  return $found if $found->{kty} eq 'RSA' && $alg =~ /^RS/;
+  return $found if $found->{kty} eq 'RSA' && $alg =~ /^(RS|PS)/;
   croak "JWT: key type '$found->{kty}' cannot be used with alg '$alg'";
 }
 
@@ -215,10 +215,14 @@ sub _verify_claims {
         croak "JWT: $claim claim re check failed" unless $value =~ $check;
       }
       elsif (ref $check eq 'CODE') {
-        croak "JWT: $claim claim check failed" unless $check->($payload->{$claim});
+        croak "JWT: $claim claim code check failed" unless $check->($payload->{$claim});
+      }
+      elsif (!ref $check) {
+        my $value = $payload->{$claim};
+        croak "JWT: $claim claim scalar check failed" unless defined $value && $value eq $check;
       }
       else {
-        croak "JWT: verify_$claim must be Regexp or CODE";
+        croak "JWT: verify_$claim must be Regexp, Scalar or CODE";
       }
     }
     else {
@@ -442,6 +446,8 @@ sub _encode_jwe {
   my $alg     = $args{alg};
   my $enc     = $args{enc};
   my $header  = $args{extra_headers} ? \%{$args{extra_headers}} : {};
+  croak "JWE: missing 'enc'" if !defined $enc;
+  croak "JWE: missing 'payload'" if !defined $payload;
   # add claims to payload
   _add_claims($payload, %args) if ref $payload eq 'HASH';
   # serialize payload
@@ -544,9 +550,10 @@ sub _sign_jws {
     my $pk = _prepare_rsa_key($key);
     $sig  = $pk->sign_message($data, $hash, 'pss', $hashlen);
   }
-  elsif ($alg =~ /^ES(256|384|512)/) { # ECDSA signatures
+  elsif ($alg =~ /^ES(256|256K|384|512)$/) { # ECDSA signatures
+    my $hash = {ES256 => 'SHA256', ES256K => 'SHA256', ES384 => 'SHA384', ES512 => 'SHA512'}->{$alg};
     my $pk = _prepare_ecc_key($key);
-    $sig  = $pk->sign_message_rfc7518($data, "SHA$1");
+    $sig  = $pk->sign_message_rfc7518($data, $hash);
   }
   elsif ($alg eq 'EdDSA') { # Ed25519 signatures
     my $pk = _prepare_ed25519_key($key);
@@ -579,8 +586,8 @@ sub _verify_jws {
     my $pk = _prepare_rsa_key($key);
     return 1 if $pk->verify_message($sig, $data, $hash, 'pss', $hashlen);
   }
-  elsif ($alg =~ /^ES(256|384|512)/) { # ECDSA signatures
-    my $hash = "SHA$1";
+  elsif ($alg =~ /^ES(256|256K|384|512)$/) { # ECDSA signatures
+    my $hash = {ES256 => 'SHA256', ES256K => 'SHA256', ES384 => 'SHA384', ES512 => 'SHA512'}->{$alg};
     my $pk = _prepare_ecc_key($key);
     return 1 if $pk->verify_message_rfc7518($sig, $data, $hash);
   }
@@ -596,6 +603,7 @@ sub _encode_jws {
   my $payload = $args{payload};
   my $alg     = $args{alg};
   my $header  = $args{extra_headers} ? \%{$args{extra_headers}} : {};
+  croak "JWS: missing 'payload'" if !defined $payload;
   croak "JWS: alg 'none' not allowed" if $alg eq 'none' && !$args{allow_none};
   # add claims to payload
   _add_claims($payload, %args) if ref $payload eq 'HASH';
@@ -683,8 +691,9 @@ sub _decode_jws {
 sub encode_jwt {
   my %args = @_;
 
+  croak "JWT: missing 'alg'" unless $args{alg};
   my $ser = $args{serialization} || 'compact';
-  if ($args{alg} =~ /^(none|EdDSA|((HS|RS|PS|ES)(512|384|256)))$/) {
+  if ($args{alg} =~ /^(none|EdDSA|(HS|RS|PS)(256|384|512)|ES(256|256K|384|512))$/) {
     ###JWS
     my ($b64u_header, $b64u_payload, $b64u_signature) = _encode_jws(%args);
     if ($ser eq 'compact') { # https://tools.ietf.org/html/rfc7515#section-7.1
@@ -700,7 +709,7 @@ sub encode_jwt {
       croak "JWT: unsupported JWS serialization '$ser'";
     }
   }
-  else {
+  elsif ($args{alg} =~ /^(dir|A(128|192|256)KW|A(128|192|256)GCMKW|PBES2-(HS256\+A128KW|HS384\+A192KW|HS512\+A256KW)|RSA-OAEP|RSA-OAEP-256|RSA1_5|ECDH-ES\+A(128|192|256)KW|ECDH-ES)$/) {
     ### JWE
     my ($b64u_header, $b64u_ecek, $b64u_iv, $b64u_ct, $b64u_tag, $b64u_aad) = _encode_jwe(%args);
     if ($ser eq 'compact') { # https://tools.ietf.org/html/rfc7516#section-7.1
@@ -729,6 +738,9 @@ sub encode_jwt {
       croak "JWT: unsupported JWE serialization '$ser'";
     }
   }
+  else {
+      croak "JWT: unexpected alg '$args{alg}'";
+  }
 }
 
 sub decode_jwt {
@@ -862,6 +874,7 @@ The value depends on the C<alg> token header value.
  ES256               public ECC key, perl HASH ref with JWK key structure,
                      a reference to SCALAR string with PEM or DER or JSON/JWK data,
                      an instance of Crypt::PK::ECC
+ ES256K              public ECC key, see ES256
  ES384               public ECC key, see ES256
  ES512               public ECC key, see ES256
  EdDSA               public Ed25519 key
@@ -1098,6 +1111,8 @@ C<CODE ref> - subroutine (with 'iss' claim value passed as argument) should retu
 
 C<Regexp ref> - 'iss' claim value has to match given regexp otherwise verification fails
 
+C<Scalar> - 'iss' claim value has to be equal to given string (since 0.029)
+
 C<undef> (default) - do not verify 'iss' claim
 
 =item verify_aud
@@ -1109,6 +1124,8 @@ C<CODE ref> - subroutine (with 'aud' claim value passed as argument) should retu
 
 C<Regexp ref> - 'aud' claim value has to match given regexp otherwise verification fails
 
+C<Scalar> - 'aud' claim value has to be equal to given string (since 0.029)
+
 C<undef> (default) - do not verify 'aud' claim
 
 =item verify_sub
@@ -1120,6 +1137,8 @@ C<CODE ref> - subroutine (with 'sub' claim value passed as argument) should retu
 
 C<Regexp ref> - 'sub' claim value has to match given regexp otherwise verification fails
 
+C<Scalar> - 'sub' claim value has to be equal to given string (since 0.029)
+
 C<undef> (default) - do not verify 'sub' claim
 
 =item verify_jti
@@ -1131,6 +1150,8 @@ C<CODE ref> - subroutine (with 'jti' claim value passed as argument) should retu
 
 C<Regexp ref> - 'jti' claim value has to match given regexp otherwise verification fails
 
+C<Scalar> - 'jti' claim value has to be equal to given string (since 0.029)
+
 C<undef> (default) - do not verify 'jti' claim
 
 =item verify_iat
@@ -1226,6 +1247,7 @@ Supported JWS algorithms:
  PS384  ...  RSA+PSS + SHA384 signature
  PS512  ...  RSA+PSS + SHA512 signature
  ES256  ...  ECDSA + SHA256 signature
+ ES256K ...  ECDSA + SHA256 signature
  ES384  ...  ECDSA + SHA384 signature
  ES512  ...  ECDSA + SHA512 signature
  EdDSA  ...  Ed25519 signature
@@ -1264,6 +1286,7 @@ A key used for token encryption (JWE) or token signing (JWS). The value depends
  ES256               private ECC key, perl HASH ref with JWK key structure,
                      a reference to SCALAR string with PEM or DER or JSON/JWK data,
                      an instance of Crypt::PK::ECC
+ ES256K              private ECC key, see ES256
  ES384               private ECC key, see ES256
  ES512               private ECC key, see ES256
  EdDSA               private Ed25519 key
@@ -1370,4 +1393,4 @@ This program is free software; you can redistribute it and/or modify it under th
 
 =head1 COPYRIGHT
 
-Copyright (c) 2015 DCIT, a.s. L<http://www.dcit.cz> / Karel Miko
+Copyright (c) 2015-2021 DCIT, a.s. L<https://www.dcit.cz> / Karel Miko
